@@ -447,18 +447,29 @@ class NmfModel:
     def get_sorted_observations(self):
         return list(self._sortedmat.index)
 
-    def get_observations_clusters_hhi(self):
+    def get_observations_clusters_hhi_1(self):
         df_obs_cl = self.get_obs_leverage_clusters()
         df_count_by_clust = df_obs_cl.groupby("Observation Cluster").size().reset_index(name="counts")
         return utl.hhi_1(df_count_by_clust["counts"].values)
 
-    def get_features_clusters(self):
+    def get_features_leverage_clusters(self):
         if not self.is_ok():
             self.test_model()
         return pd.DataFrame(index=self._fnames, columns=["Feature Cluster"], data=self._predictor["HC"])
 
+    def get_features_kmeans_clusters(self, ngroups=0):
+        """
+        Attribue à chaque ligne de H un cluster dans [1, ngroups], à partir d'une décomposition en K-Means de H
+        """
+        if not self.is_ok():
+            self.test_model()
+        hmat = self.get_h()
+        ngr = ngroups if ngroups > 0 else self._ncomp
+        km = KMeans(ngr)
+        return pd.DataFrame(index=self._fnames, columns=["Feature Cluster"], data=km.fit_predict(hmat) + 1)
+
     def get_features_clusters_hhi_1(self):
-        df_feat_cl = self.get_features_clusters()
+        df_feat_cl = self.get_features_leverage_clusters()
         df_count_by_clust = df_feat_cl.groupby("Feature Cluster").size().reset_index(name="counts")
         return utl.hhi_1(df_count_by_clust["counts"].values)
 
@@ -473,9 +484,11 @@ class NmfModel:
         mat_w_clust = mat_w_clust.sort_values(by="OBJ_CLUSTER", axis=0)
         self._sortedmat = mat_w_clust.drop("OBJ_CLUSTER", axis=1)
 
-    def get_entropies_by_cluster(self, nbins=10):
-        l_clusters = self.get_obs_leverage_clusters()
-        c_clusters = self.get_features_clusters()
+    def get_entropies_by_cluster(self, nbins=10, clustering_method='kmeans'):
+        l_clusters = self.get_obs_kmeans_clusters() if clustering_method.lower() == 'kmeans' else \
+            self.get_obs_leverage_clusters()
+        c_clusters = self.get_features_kmeans_clusters() if clustering_method.lower() == "kmeans" else \
+            self.get_features_leverage_clusters()
         return utl.relative_entropy_by_cluster(self._sortedmat, l_clusters, c_clusters, nbins)
 
     def get_avg_diag_entropies(self, nbins: int = 3,
@@ -632,24 +645,47 @@ def compare_nmf_fs_to_random(train_set: Union[np.ndarray, pd.DataFrame],
     score_test_reduced = svc_model_reduced.score(df_x_test_reduced, test_labels)
 
 
-def scree_plot(mat: Union[np.ndarray, pd.DataFrame],
-               ncmin: int, ncmax: int):
+def generate_scree_plot(mat: Union[np.ndarray, pd.DataFrame],
+                        ncmin: int, ncmax: int):
     columns = []
     ncomp = ncmin
     nmfmodel = NmfModel(mat=mat, ncomp=ncomp, regularization="components")
+    n, f = mat.shape
     df_scree_plot = pd.DataFrame(columns=columns)
     for ncomp in range(ncmin, ncmax+1):
         nmfmodel.set_ncomp(ncomp)
         line = pd.DataFrame(index=[ncomp], columns=columns)
         err = line.loc[ncomp, "Error"] = nmfmodel.get_precision()
         vol = line.loc[ncomp, "Volume"] = nmfmodel.get_volume()
+        line.loc[ncomp, "Tried_ncomp"] = ncomp
+        line.loc[ncomp, "Obs_feat_ratio"] = float(n) / float(f)
         line.loc[ncomp, "Err_Vol_Ratio"] = err / vol if vol > 0 else 1e6
-        line.loc[ncomp, "W_entropy"] = nmfmodel.get_specific_cluster_contribution()
+        line.loc[ncomp, "W_clust_hhi_1"] = nmfmodel.get_observations_clusters_hhi_1()
         line.loc[ncomp, "H_clust_hhi_1"] = nmfmodel.get_features_clusters_hhi_1()
-        df_scree_plot = pd.concat([df_scree_plot, line], axis=0)
+        line.loc[ncomp, "Avg_relative_entropy"] = nmfmodel.get_avg_diag_entropies(nbins=5,
+                                                                                  size_weighted=True,
+                                                                                  relative=True).iloc[0, 0]
+    df_scree_plot = pd.concat([df_scree_plot, line], axis=0)
     return df_scree_plot
 
 
+def add_miniscree_plots_from_scree_plot(df_examples: pd.DataFrame,
+                                        scree_plot: pd.DataFrame,
+                                        known_ncomp: int,
+                                        window_size: int = 6):
+
+    nb_examples, _ = df_examples.shape
+    for ncomp in scree_plot.index:
+        if ncomp + window_size - 1 <= scree_plot.index[-1]:
+            example = pd.DataFrame(index=[nb_examples], columns=scree_plot.columns)
+            position = 0 if known_ncomp <= ncomp else window_size-1 if known_ncomp >= ncomp + window_size - 1 \
+                else known_ncomp - ncomp
+            example.loc[nb_examples, :] = scree_plot.loc[ncomp, :]
+            example.loc[nb_examples, "Position"] = position
+            nb_examples += 1
+            df_examples = pd.concat([df_examples, example], axis=0)
+    return df_examples
+
 
 
 
