@@ -698,18 +698,28 @@ def generate_orthogonal_vector(v: object) -> pd.DataFrame:
     return w
 
 
-def generate_vectors_bouquet(vpivot, bouquet_size: int, min_corr: float) -> pd.DataFrame:
+def generate_vectors_bouquet(vpivot, bouquet_size: int, min_corr: float,
+                             positive: bool = True,
+                             col_shift: int = 0,
+                             row_shift: int = 0) -> pd.DataFrame:
 
     vpivot = pd.DataFrame(vpivot)
-    df_bouquet = pd.DataFrame(index=vpivot.index, columns=range(bouquet_size))
+    nr, _ = vpivot.shape
+    df_bouquet = pd.DataFrame(index=range(row_shift, row_shift+nr),
+                              columns=range(col_shift, col_shift+bouquet_size))
     df_bouquet.fillna(value=0.0, inplace=True)
     epsilon = (1 - min_corr)**0.5
     n2v = norm_n2(vpivot)
     for icol in range(bouquet_size):
         w = generate_orthogonal_vector(vpivot)
         n2w = norm_n2(w)
-        vals = vpivot + epsilon * n2v / n2w * pd.DataFrame(w)
-        df_bouquet.loc[:, icol] = vals
+        rescaled_w = epsilon * n2v / n2w * pd.DataFrame(w)
+        vals = vpivot.values + rescaled_w.values
+        df_bouquet.loc[:, col_shift+icol] = vals
+    if positive:
+        minval = np.min(np.min(df_bouquet))
+        if minval < 0:
+            df_bouquet -= minval
     return df_bouquet
 
 
@@ -731,27 +741,63 @@ def generate_clusters_centroids(ndim: int, max_corr: float):
     return df_centroids
 
 
-def generate_clusterized_vectors(veclen: int,
-                                 nclusters: int,
-                                 nbvect: int,
-                                 max_inter_corr: float,
-                                 min_intra_corr: float):
-
-    pass
-
-
 def generate_integer_partition(ntot: int, nbpart: int, nmin: int = 1):
 
     assert(ntot >= nbpart*nmin)
     rng = np.random.default_rng()
-    cuts = np.sort(rng.choice(ntot, nbpart))
-    partsize = ntot / nbpart
-    res = [partsize] * nbpart
-    if ntot % nbpart > 0:
-        res[nbpart - 1] = partsize + ntot % nbpart
+    cuts = np.sort(rng.choice(ntot, nbpart, replace=False))
+    res = []
+    zum = 0
+    defect = 0
+    for i, cut in enumerate(cuts):
+        diff = cut if i == 0 else cut - cuts[i-1]
+        if diff < nmin:
+            defect += nmin - diff
+            val = nmin
+        elif defect == 0:
+            val = diff
+        elif diff >= nmin + defect:
+            val = diff - defect
+            defect = 0
+        else:
+            val = diff
+        res.append(val)
+        zum += val
+    if zum < ntot:
+        res[-1] += ntot-zum
+    # partsize = ntot / nbpart
+    # res = [partsize] * nbpart
     return res
 
 
+def generate_clusterized_vectors(veclen: int,
+                                 nclusters: int,
+                                 nbvect: int,
+                                 clusterminsize: int,
+                                 max_inter_corr: float,
+                                 min_intra_corr: float,
+                                 random_norms: bool = True):
+
+    clustersizes = generate_integer_partition(ntot=nbvect, nbpart=nclusters, nmin=clusterminsize)
+    centroids = generate_clusters_centroids(ndim=veclen, max_corr=max_inter_corr)
+    df_res = pd.DataFrame(index=range(veclen))
+    # factors = np.random.exponential(size=nclusters) if random_norms else np.zeros(shape=nclusters)
+    factors = np.random.lognormal(mean=0, sigma=1, size=nclusters)
+    row_shift = 0
+    col_shift = 0
+    for icluster, csize in enumerate(clustersizes):
+        centroid = centroids.iloc[:, icluster]
+        df_bouquet = generate_vectors_bouquet(vpivot=centroid,
+                                              bouquet_size=int(csize),
+                                              min_corr=min_intra_corr,
+                                              col_shift=col_shift
+                                              )
+        df_bouquet *= 1 + factors[icluster]
+        df_res = pd.concat([df_res, df_bouquet], axis=1)
+        col_shift += int(csize)
+    return df_res
+
+
 def build_correspondence_map(df_clusters):
 
     assert len(df_clusters.columns) >= 2
@@ -775,3 +821,5 @@ def plot_correspondence_map(df_clusters, path=None):
 
     df_map = build_correspondence_map(df_clusters)
     plot_values(df_map, xticklabels=True, path=path)
+
+
