@@ -242,19 +242,22 @@ def generate_nmf_reconstruction(n_comp: int,
                                 max_inter_corr: float,
                                 h_avg_clust_size: int,
                                 w_avg_clust_size: int,
-                                epsilon: float):
+                                epsilon: float,
+                                random_norms: bool = True):
     cl_vects_H = utl.generate_clusterized_vectors(veclen=n_comp,
                                                   nclusters=n_comp,
                                                   nbvect=n_comp * h_avg_clust_size,
                                                   clusterminsize=int(h_avg_clust_size/2),
                                                   max_inter_corr=max_inter_corr,
-                                                  min_intra_corr=min_intra_corr)
+                                                  min_intra_corr=min_intra_corr,
+                                                  random_norms=random_norms)
     cl_vects_W = utl.generate_clusterized_vectors(veclen=nb_clusters,
                                                   nclusters=nb_clusters,
                                                   nbvect=nb_clusters * w_avg_clust_size,
                                                   clusterminsize=int(w_avg_clust_size/2),
                                                   max_inter_corr=max_inter_corr,
-                                                  min_intra_corr=min_intra_corr)
+                                                  min_intra_corr=min_intra_corr,
+                                                  random_norms=random_norms)
     cl_vects_W = cl_vects_W.T
     cl_vects_M = cl_vects_W @ cl_vects_H
     noise = np.random.normal(loc=0.0, scale=epsilon, size=cl_vects_M.shape)
@@ -273,28 +276,37 @@ if __name__ == "__main__":
     data_path = imputation_paper_path / os.getenv("data_subpath")
     res_path = imputation_paper_path / os.getenv("results_subpath")
 
-    # df_mini_scree_plots = pd.read_csv(res_path / "random_nmf" / f"mini_scree_plots.csv")
-    df_mini_scree_plots = pd.DataFrame(index=[], columns=[])
+    train_f_name = f"mini_scree_plots.csv"
 
-    for itrial in range(40):
+    if os.path.isfile(res_path / "random_nmf" / train_f_name):
+        df_mini_scree_plots = pd.read_csv(res_path / "random_nmf" / train_f_name)
+    else:
+        df_mini_scree_plots = pd.DataFrame(index=[], columns=[])
+
+    # train_f_name = "foo_scree.csv"
+    random_norms = True
+    for itrial in range(20):
         nb_clusters = np.random.randint(low=5, high=50)
-        min_corr = np.random.uniform(low=0.85, high=0.95)
-        max_corr = np.random.uniform(low=0.05, high=0.15)
+        min_corr = np.random.uniform(low=0.80, high=0.95)
+        max_corr = np.random.uniform(low=0.05, high=0.20)
         h_avg_csize = np.random.randint(low=10, high=20)
-        w_avg_csize = np.random.randint(low=10, high=50)
-        eps = np.random.uniform(low=0.05, high=0.20)
+        w_avg_csize = np.random.randint(low=10, high=100)
+        # eps = np.random.uniform(low=0.05, high=0.25)
+        eps = 0.0
         generated_M = generate_nmf_reconstruction(n_comp=nb_clusters,
                                                   min_intra_corr=min_corr,
                                                   max_inter_corr=max_corr,
                                                   h_avg_clust_size=h_avg_csize,
                                                   w_avg_clust_size=w_avg_csize,
-                                                  epsilon=eps)
+                                                  epsilon=eps,
+                                                  random_norms=random_norms)
+        rnstr = "RandNorms" if random_norms else "ConstNorms"
         generated_M.to_csv(res_path / "random_nmf" / f"M_nc{nb_clusters}_corrmin{round(min_corr,2)}_"
-                                       f"corrmax{round(max_corr,2)}_noise{round(eps,2)}.csv",
+                                       f"corrmax{round(max_corr,2)}_noise{round(eps,2)}_{rnstr}.csv",
                            float_format="%.4f")
-        df_scree_plot = nmf.generate_scree_plot(generated_M, ncmin=5, ncmax=50)
-        df_mini_scree_plots = nmf.add_examples_from_scree_plot(df_mini_scree_plots, df_scree_plot, nb_clusters)
-        df_mini_scree_plots.to_csv(res_path / "random_nmf" / f"mini_scree_plots.csv")
+        df_scree_plot = nmf.generate_scree_plot(generated_M, ncmin=5, ncmax=40)
+        df_mini_scree_plots = nmf.add_miniscree_plots_from_scree_plot(df_mini_scree_plots, df_scree_plot, nb_clusters)
+        df_mini_scree_plots.to_csv(res_path / "random_nmf" / train_f_name)
 
     do_read_mat = False
     if do_read_mat:
